@@ -65,7 +65,6 @@ label mappers.
 
 
 """
-from __future__ import absolute_import, division, unicode_literals, print_function
 import warnings
 import numpy as np
 from astropy.modeling.core import Model
@@ -80,6 +79,7 @@ __all__ = ['LabelMapperArray', 'LabelMapperDict', 'LabelMapperRange', 'RegionsSe
 
 
 def get_unique_regions(regions):
+    regions = np.asarray(regions)
     if isinstance(regions, np.ndarray):
         unique_regions = np.unique(regions).tolist()
 
@@ -186,7 +186,6 @@ class LabelMapperArray(_LabelMapper):
             _no_label = 0
         else:
             _no_label = ""
-
         super(LabelMapperArray, self).__init__(mapper, _no_label, name=name, **kwargs)
 
     def evaluate(self, *args):
@@ -226,11 +225,11 @@ class LabelMapperArray(_LabelMapper):
         Examples
         --------
         >>> regions = {1: [[795, 970], [2047, 970], [2047, 999], [795, 999], [795, 970]],
-                       2: [[844, 1067], [2047, 1067], [2047, 1113], [844, 1113], [844, 1067]],
-                       3: [[654, 1029], [2047, 1029], [2047, 1078], [654, 1078], [654, 1029]],
-                       4: [[772, 990], [2047, 990], [2047, 1042], [772, 1042], [772, 990]]
-                      }
-        >>> mapper = selector.LabelMapperArray.from_vertices((2400, 2400), regions)
+        ...            2: [[844, 1067], [2047, 1067], [2047, 1113], [844, 1113], [844, 1067]],
+        ...            3: [[654, 1029], [2047, 1029], [2047, 1078], [654, 1078], [654, 1029]],
+        ...            4: [[772, 990], [2047, 990], [2047, 1042], [772, 1042], [772, 990]]
+        ...           }
+        >>> mapper = LabelMapperArray.from_vertices((2400, 2400), regions)
 
         """
         labels = np.array(list(regions.keys()))
@@ -276,11 +275,21 @@ class LabelMapperDict(_LabelMapper):
 
     def __init__(self, inputs, mapper, inputs_mapping=None, atol=10**-8, name=None, **kwargs):
         self._atol = atol
-        self._inputs = inputs
         _no_label = 0
+        self._inputs = inputs
         if not all([m.n_outputs == 1 for m in mapper.values()]):
             raise TypeError("All transforms in mapper must have one output.")
-        super(LabelMapperDict, self).__init__(mapper, _no_label, inputs_mapping, name=name, **kwargs)
+        self._input_units_strict = {key: False for key in self._inputs}
+        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
+        super(LabelMapperDict, self).__init__(mapper, _no_label, inputs_mapping,
+                                              name=name, **kwargs)
+
+    @property
+    def inputs(self):
+        """
+        The name(s) of the input variable(s) on which a model is evaluated.
+        """
+        return self._inputs
 
     @property
     def atol(self):
@@ -290,10 +299,6 @@ class LabelMapperDict(_LabelMapper):
     def atol(self, val):
         self._atol = val
 
-    @property
-    def inputs(self):
-        return self._inputs
-
     def evaluate(self, *args):
         shape = args[0].shape
         args = [a.flatten() for a in args]
@@ -359,7 +364,17 @@ class LabelMapperRange(_LabelMapper):
         _no_label = 0
         if not all([m.n_outputs == 1 for m in mapper.values()]):
             raise TypeError("All transforms in mapper must have one output.")
-        super(LabelMapperRange, self).__init__(mapper, _no_label, inputs_mapping, name=name, **kwargs)
+        self._input_units_strict = {key: False for key in self._inputs}
+        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
+        super(LabelMapperRange, self).__init__(mapper, _no_label, inputs_mapping,
+                                               name=name, **kwargs)
+
+    @property
+    def inputs(self):
+        """
+        The name(s) of the input variable(s) on which a model is evaluated.
+        """
+        return self._inputs
 
     @staticmethod
     def _has_overlapping(ranges):
@@ -376,7 +391,7 @@ class LabelMapperRange(_LabelMapper):
         l = np.array(l)
         start = np.roll(l[:, 0], -1)
         end = l[:, 1]
-        if any((end - start)[:-1]  > 0) or any(start[-1] > end):
+        if any((end - start)[:-1] > 0) or any(start[-1] > end):
             return True
         else:
             return False
@@ -408,10 +423,6 @@ class LabelMapperRange(_LabelMapper):
         else:
             return ind.item()
 
-    @property
-    def inputs(self):
-        return self._inputs
-
     def evaluate(self, *args):
         shape = args[0].shape
         args = [a.flatten() for a in args]
@@ -432,7 +443,7 @@ class LabelMapperRange(_LabelMapper):
             temp[nan_ind] = np.nan
             temp = np.where(np.logical_or(temp <= val_range[0],
                                           temp >= val_range[1]),
-                                          np.nan, temp)
+                            np.nan, temp)
             ind = ~np.isnan(temp)
 
             if ind.any():
@@ -486,7 +497,8 @@ class RegionsSelector(Model):
 
         if " " in selector.keys() or 0 in selector.keys():
             raise ValueError('"0" and " " are not allowed as keys.')
-
+        self._input_units_strict = {key: False for key in self._inputs}
+        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
         super(RegionsSelector, self).__init__(n_models=1, name=name, **kwargs)
 
     def set_input(self, rid):
@@ -559,13 +571,6 @@ class RegionsSelector(Model):
     def undefined_transform_value(self, value):
         self._undefined_transform_value = value
 
-    @property
-    def inputs(self):
-        """
-        The name(s) of the input variable(s) on which a model is evaluated.
-        """
-        return self._inputs
-
     @property
     def outputs(self):
         """The name(s) of the output(s) of the model."""
@@ -575,6 +580,13 @@ class RegionsSelector(Model):
     def selector(self):
         return self._selector
 
+    @property
+    def inputs(self):
+        """
+        The name(s) of the input variable(s) on which a model is evaluated.
+        """
+        return self._inputs
+
 
 class LabelMapper(_LabelMapper):
     """
@@ -602,17 +614,26 @@ class LabelMapper(_LabelMapper):
 
     def __init__(self, inputs, mapper, no_label=np.nan, inputs_mapping=None, name=None, **kwargs):
         self._no_label = no_label
-        self.inputs = inputs
+        self._inputs = inputs
         self.outputs = tuple(['x{0}'.format(ind) for ind in list(range(mapper.n_outputs))])
         if isinstance(inputs_mapping, tuple):
             inputs_mapping = astmodels.Mapping(inputs_mapping)
         elif inputs_mapping is not None and not isinstance(inputs_mapping, astmodels.Mapping):
-            raise TypeError("inputs-mapping must be an instance of astropy.modeling.Mapping.")
+            raise TypeError("inputs_mapping must be an instance of astropy.modeling.Mapping.")
 
         self._inputs_mapping = inputs_mapping
         self._mapper = mapper
+        self._input_units_strict = {key: False for key in self._inputs}
+        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
         super(_LabelMapper, self).__init__(name=name, **kwargs)
 
+    @property
+    def inputs(self):
+        """
+        The name(s) of the input variable(s) on which a model is evaluated.
+        """
+        return self._inputs
+
     @property
     def mapper(self):
         return self._mapper
